/*
 * Copyright (C) 1994	Remy Card (card@masi.ibp.fr)
 *			Laboratoire MASI - Institut Blaise Pascal
 *			Universite Pierre et Marie Curie (Paris VI)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 * main.c	- test server running under Mach
 *
 */

#include <mach.h>
#include <device/device.h>
#include <mach/sys_reboot.h>

mach_port_t	host_port;
mach_port_t	device_port;
mach_port_t	console_port;

void console_init (void)
{
	kern_return_t		rc;

	/*
	 * Open the console
	 */
	rc = device_open (device_port, D_READ | D_WRITE, "console", &console_port);
	/*
	 * Call `printf_init' to enable `printf'
	 */
	printf_init (device_port);
}

char readchar (void)
{
	char			ch;
	unsigned int		rc;

	device_read_inband (console_port, 0, 0, 1, &ch, &rc);
	if (rc != 1)
		printf ("hey, nothing to read");
	return ch;
}

main (int argc, char **argv)
{
	vm_statistics_data_t	vmsd;
	kern_return_t		rc;
	char			c;

	/*
	 * Get the master ports from the bootstrap task
	 */
	get_privileged_ports (&host_port, &device_port);

	/*
	 * Initialize the console
	 */
	console_init ();

	/*
	 * Print a `hello' message
	 */
	printf ("Test server running on top of Mach!\n");
	printf ("Hit a key to start:");
	(void) readchar ();
	printf ("\n");

	/*
	 * Display the arguments
	 * arg #0: program name
	 * arg #1: bootflags
	 * arg #2: root partition name
	 * arg #3: server(s) directory
	 */
	printf ("\nArguments:\n");
	printf ("  Program name      : %s\n", argv[0]);
	printf ("  Boot flags        : %s\n", argv[1]);
	printf ("  Root partition    : %s\n", argv[2]);
	printf ("  Servers directory : %s\n", argv[3]);

	do
	{
		/*
		 * Get the Virtual Memory statistics
		 */
		rc = vm_statistics (mach_task_self (), &vmsd);
		if (rc)
			printf ("vm_statistics failed (error %d)\n", rc);
		else
		{
			printf ("\nVM statistics:\n");
			printf ("  Page size            : %d\n", vmsd.pagesize);
			printf ("  Free pages           : %d\n", vmsd.free_count);
			printf ("  Active pages         : %d\n", vmsd.active_count);
			printf ("  Inactive pages       : %d\n", vmsd.inactive_count);
			printf ("  Wired pages          : %d\n", vmsd.wire_count);
			printf ("  Zero fill pages      : %d\n", vmsd.zero_fill_count);
			printf ("  Reactivated pages    : %d\n", vmsd.reactivations);
			printf ("  Pageins              : %d\n", vmsd.pageins);
			printf ("  Pageouts             : %d\n", vmsd.pageouts);
			printf ("  Page faults          : %d\n", vmsd.faults);
			printf ("  Copy on write faults : %d\n", vmsd.cow_faults);
		}

		printf ("Hit `r' to reboot, `d' to call the debugger: ");
		c = readchar ();
		printf ("\n");
		switch (c)
		{
			case 'd':
				host_reboot (host_port, RB_DEBUGGER);
				break;
			case 'r':
				break;
			default:
				printf ("Invalid key!\n");
		}
	} while (c != 'r');

	/*
	 * Now, reboot Mach
	 */

	host_reboot (host_port, RB_AUTOBOOT);

	printf ("Reboot failed !!!\n");
	while (1);
}
